/* 
block.c 
ALICE V 2.0, January 1993
library functions for block operations 
(C) Ralf Kaufmann 1993 
used C compiler: Pure C 
*/ 
 
/* 
functions contained in this module: 
block_set 
copy_block 
delete_block 
paste_block 
paste_include_file
init_buffer 
init_include_buffer 
exist_buffer 
clear_block_flags
*/ 
 
 
/* Includefiles */ 
 
# include <ctype.h> 
# include <math.h> 
# include <stdio.h> 
# include <stdlib.h> 
# include <string.h> 
 
# include <aes.h> 
# include <tos.h> 
# include <vdi.h> 
 
 
# include "language.h" 
# include "prvdef.h" 
# include "mewidi.h" 
# include "messages.h" 
# include "scancode.h" 
# include "structs.h" 
# include "extdef.h" 
# include "rsc.h" 
 
/* References to global data defined elsewhere  */ 
 
extern  int         ap_id, handle; 
extern  int         bzeich, hzeich, gbl_clip[]; 
extern  WIND_DATA   windows[]; 
extern  BUFFER      buffers[]; 
extern  BUFFER      include_buffer; 
extern  OBJECT      *menu_tree; 
extern  OBJECT      *dial_about, *dial_help, *dial_print, *dial_tab; 
extern  OBJECT      *dial_selebuf; 
extern  int         prgquit, prgexit, page_length, left_margin; 
extern  int         tab_size, buffer_index; 
extern  char        gbl_short_line_buf[];
extern  char        gbl_line_buf1[];
extern  char        gbl_line_buf2[];
 
 
/**********************/ 
/* Start of routines  */ 
/**********************/ 
 
 
 
 
int   block_set (w_index)   /* checks whether a proper block is   */ 
                            /* defined for window w_index         */ 
int   w_index; 
{ 

  if (w_index < 0) return (FALSE);
  if (windows[w_index].block.cs_x == -1) return (FALSE); 
  if (windows[w_index].block.cs_y == -1) return (FALSE); 
  if (windows[w_index].block.ce_x == -1) return (FALSE); 
  if (windows[w_index].block.ce_y == -1) return (FALSE); 
 
  if (((windows[w_index].block.cs_y == windows[w_index].block.ce_y) && 
       (windows[w_index].block.cs_x < windows[w_index].block.ce_x)) || 
      (windows[w_index].block.cs_y < windows[w_index].block.ce_y)) 
  { 
    return (TRUE); 
  } 
  else 
  { 
    return (FALSE); 
  } 
}                                         /* end function block_set */ 
 
 
 
 
int   copy_block (w_index) 
int   w_index;            /* index into global "windows" structure  */ 
{ 
  long          i, end_x; 
  long          error; 
  int           b_index;                            /* buffer index */ 
  long          start_x, start_y, wind_h, wind_w; 
  long          doc_w, doc_h; 
  long          curs_x, curs_y, cursor_visible; 
  LINE_DESC     *p, *q; 
  TEDINFO     *ted_info; 
 
 
  b_index = buffer_index;
  init_buffer (b_index); 
  error = FALSE; 
 
  for (i = 1, p = windows[w_index].head; 
       i < windows[w_index].block.cs_y; i++) 
  { 
    p = p->next; 
  } 
 
  for (i = windows[w_index].block.cs_y; 
       i <= windows[w_index].block.ce_y; i++) 
  {                           /* 1. for start */ 
    if (p == NULL) break; 
    if (i == windows[w_index].block.cs_y) 
    { 
      start_x = windows[w_index].block.cs_x - 1; 
      start_x = max (start_x, (long) 0); 
    } 
    else 
    { 
      start_x = 0; 
    } 
    memset (gbl_short_line_buf, (int) 0, (size_t) (MAX_LINE-1));
    strcpy (gbl_short_line_buf, (p->line) + start_x); 
                                  /* copy to temporary line buffer  */ 
 
    if (i == windows[w_index].block.ce_y) 
    { 
      if (i == windows[w_index].block.cs_y) 
      { 
        end_x = windows[w_index].block.ce_x - 
                  windows[w_index].block.cs_x; 
      } 
      else 
      { 
        end_x = windows[w_index].block.ce_x - 1; 
        if (end_x == 0) break;            /* quit the for loop as   */ 
                                          /* there is nothing to do */ 
                                          /* in the last line       */ 
      } 
      gbl_short_line_buf[end_x] = '\0'; 
    } 
 
    if ((q = create_list_element ()) == NULL) 
    { 
      error = TRUE; 
      break;                /* leave the for loop */ 
    } 
 
    if ((q->line = (char *) malloc (MAX_LINE + 1)) == NULL) 
    { 
      error = TRUE; 
      free ((LINE_DESC *) q); 
      break;                /* leave the for loop */ 
    } 
    memset (q->line, (int) 0, (size_t) (MAX_LINE-1)); 
    strcpy (q->line, gbl_short_line_buf); /* here is where the copy */ 
                                          /* work gets done         */ 
 
    q->len  = (int) strlen (q->line); 
    q->size = MAX_LINE; 
 
    add_element_buffer (q, b_index);    /* add q to end of list    */ 
    p = p->next;                        /* next string             */ 
  }                                     /* 1. for end              */ 
 
  if (error) 
  { 
    put_message (OUT_OF_MEMORY); 
    init_buffer (b_index); 
    return (FALSE); 
  } 
 
  windows[w_index].curs_x  = windows[w_index].block.cs_x; 
  windows[w_index].curs_xs = windows[w_index].curs_x; 
  windows[w_index].curs_y  = windows[w_index].block.cs_y; 
 
  start_x = windows[w_index].doc_x / bzeich; 
  start_y = windows[w_index].doc_y / hzeich; 
  wind_w  = windows[w_index].work.g_w / bzeich; 
  wind_h  = windows[w_index].work.g_h / hzeich; 
 
  cursor_visible = TRUE; 
  if ((windows[w_index].curs_y < start_y + 1)       || 
      (windows[w_index].curs_y > start_y + wind_h)  || 
      (windows[w_index].curs_x < start_x + 1)       || 
      (windows[w_index].curs_x > start_x + wind_w)) 
  { 
      cursor_visible = FALSE; 
  } 
  if (!cursor_visible) 
  { 
    curs_y  = (windows[w_index].curs_y - 1) * hzeich; 
    doc_h   = windows[w_index].work.g_h; 
    windows[w_index].doc_y = 
      max ((long) (curs_y - doc_h/2), (long) 0); 
 
    curs_x  = (windows[w_index].curs_x - 1) * bzeich; 
    doc_w   = windows[w_index].work.g_w; 
    windows[w_index].doc_x = 
      max ((long) (curs_x - doc_w/2), (long) 0); 
  } 

  switch (buffer_index)
  {
    case 0: 
      ted_info = 
            dial_selebuf[BUFNAM1].ob_spec.tedinfo; 
      break;

    case 1: 
      ted_info = 
            dial_selebuf[BUFNAM2].ob_spec.tedinfo; 
      break;

    case 2: 
      ted_info = 
            dial_selebuf[BUFNAM3].ob_spec.tedinfo; 
      break;

    case 3: 
      ted_info = 
            dial_selebuf[BUFNAM4].ob_spec.tedinfo; 
      break;

    case 4: 
      ted_info = 
            dial_selebuf[BUFNAM5].ob_spec.tedinfo; 
      break;

    case 5: 
      ted_info = 
            dial_selebuf[BUFNAM6].ob_spec.tedinfo; 
      break;

    case 6: 
      ted_info = 
            dial_selebuf[BUFNAM7].ob_spec.tedinfo; 
      break;

    case 7: 
      ted_info = 
            dial_selebuf[BUFNAM8].ob_spec.tedinfo; 
      break;

    case 8: 
      ted_info = 
            dial_selebuf[BUFNAM9].ob_spec.tedinfo; 
      break;

    case 9: 
      ted_info = 
            dial_selebuf[BUFNAM10].ob_spec.tedinfo; 
      break;

    default:  
      say ("Nasty buffer error");
      break;

  }
  strcpy (ted_info->te_ptext, BUF_IN_USE);
                                    /* indicate buffer is in use  */
  return (TRUE); 
}                                      /* end function copy_block  */ 
 
 
 
 
void  delete_block (w_index) 
int   w_index;           /* index into global "windows" structure  */ 
{ 
  long          i; 
  long          start_x, start_y, wind_h, wind_w; 
  long          doc_w, doc_h; 
  long          curs_x, curs_y, cursor_visible; 
  LINE_DESC     *p, *next; 
 
 
 
  for (i = 1, p = windows[w_index].head; 
       i < windows[w_index].block.cs_y; i++) 
  { 
    p = p->next; 
  } 
 
  for (i = windows[w_index].block.cs_y; 
       i <= windows[w_index].block.ce_y; i++) 
  {                                                 /* 1. for start */ 
    if (p == NULL) break; 
    next = p->next; 
 
    if (i == windows[w_index].block.cs_y)           /* first line   */ 
    { 
      if (windows[w_index].block.ce_y > windows[w_index].block.cs_y) 
      { 
        strcpy ((p->line) + windows[w_index].block.cs_x - 1, "\n"); 
        p->len = (int) strlen (p->line); 
      } 
      else                          /* first line is the last line  */ 
      { 
        strcpy ((p->line) + windows[w_index].block.cs_x - 1, 
                   (p->line) + windows[w_index].block.ce_x - 1); 
        p->len = (int) strlen (p->line); 
        if (i == windows[w_index].curs_y) 
        { 
          windows[w_index].curs_x -= windows[w_index].block.ce_x - 
                                      windows[w_index].block.cs_x; 
          windows[w_index].curs_x = 
            max ((long) (windows[w_index].curs_x), (long) 1); 
        } 
        windows[w_index].curs_xs = windows[w_index].curs_x; 
        break;                                      /* that was all */ 
      } 
    } 
    else if (i == windows[w_index].block.ce_y)        /* last line  */ 
    { 
      strcpy ((p->line), 
                 (p->line) + windows[w_index].block.ce_x - 1); 
      p->len = (int) strlen (p->line); 
 
      /* pull up the last line if there is enough space */ 
 
      if (p->prev->len + p->len < p->prev->size - 1) 
      { 
        strcpy (p->prev->line + p->prev->len - 1, p->line); 
        p->prev->len = (int) strlen (p->prev->line); 
        free ((char *) p->line); 
        p->prev->next = p->next;                    /* may be NULL  */ 
        if (p->next) p->next->prev = p->prev; 
        free ((LINE_DESC *) p); 
      } 
    } 
    else      /* any other line between the first and the last line */ 
    { 
      free ((char *) p->line); 
      p->prev->next = p->next;                      /* may be NULL  */ 
      if (p->next) p->next->prev = p->prev; 
      free ((LINE_DESC *) p); 
    } 
    p = next;                                       /* next string  */ 
  }                                                 /* 1. for end   */ 
 
  windows[w_index].curs_x  = windows[w_index].block.cs_x; 
  windows[w_index].curs_xs = windows[w_index].curs_x; 
  windows[w_index].curs_y  = windows[w_index].block.cs_y; 
 
  start_x = windows[w_index].doc_x / bzeich; 
  start_y = windows[w_index].doc_y / hzeich; 
  wind_w  = windows[w_index].work.g_w / bzeich; 
  wind_h  = windows[w_index].work.g_h / hzeich; 
 
  cursor_visible = TRUE; 
  if ((windows[w_index].curs_y < start_y + 1)       || 
      (windows[w_index].curs_y > start_y + wind_h)  || 
      (windows[w_index].curs_x < start_x + 1)       || 
      (windows[w_index].curs_x > start_x + wind_w)) 
  { 
      cursor_visible = FALSE; 
  } 
  if (!cursor_visible) 
  { 
    curs_y  = (windows[w_index].curs_y - 1) * hzeich; 
    doc_h   = windows[w_index].work.g_h; 
    windows[w_index].doc_y = 
      max ((long) (curs_y - doc_h/2), (long) 0); 
 
    curs_x  = (windows[w_index].curs_x - 1) * bzeich; 
    doc_w   = windows[w_index].work.g_w; 
    windows[w_index].doc_x = 
      max ((long) (curs_x - doc_w/2), (long) 0); 
  } 
  return; 
}                                     /* end function delete_block  */ 
 
 
 
 
int   paste_block (w_index) 
int   w_index;            /* index into global "windows" structure  */ 
{ 
  long          i; 
  long          error; 
  int           b_index;                            /* buffer index */ 
  long          start_x, start_y, wind_h, wind_w; 
  long          doc_w, doc_h; 
  long          curs_x, curs_y, cursor_visible; 
  LINE_DESC     *p, *q, *next_b, *new; 
 
 
  b_index = buffer_index;
  error = FALSE; 
 
  for (i = 1, p = windows[w_index].head; 
       i < windows[w_index].curs_y; i++) 
  { 
    p = p->next; 
  } 
 
  start_x = windows[w_index].curs_x - 1; 
  start_x = max ((long) 0, start_x);
  memset (gbl_line_buf1, (int) 0, (size_t) (3*MAX_LINE-1));
  strcpy (gbl_line_buf1, (p->line) + start_x);  
                                          /* save rest of line    */
  strcpy (p->line + start_x, "\n");       /* truncate rest of line  */
  p->len = (int) strlen (p->line); 
  memset (p->line + p->len + 1, 
          (int) 0, 
          (size_t) (p->size - p->len - 2));
  
  /* deal with the first line seperately  */ 
 
  q = buffers[b_index].head; 
  if (p->len + q->len < p->size - 2)     /* enough space for copy ? */ 
  { 
    strcpy (p->line + start_x, q->line); 
    p->len = (int) strlen (p->line); 
  } 
  else 
  { 
    if ((new = create_list_element ()) == NULL) 
    { 
      error = TRUE; 
    } 
    if ((new->line = (char *) malloc (MAX_LINE + 1)) == NULL) 
    { 
      error = TRUE; 
      free ((LINE_DESC *) new); 
    } 
    memset (new->line, (int) 0, (size_t) (MAX_LINE-1));  
    strcpy (new->line, q->line);        /* here the paste work     */ 
                                        /* gets done               */ 
    new->len    = (int) strlen (new->line); 
    new->size   = MAX_LINE; 
    new->prev   = p; 
    new->next   = p->next; 
    if (p->next) p->next->prev = new; 
    p->next = new; 
    p = new; 
  } 
  if (error) 
  { 
    put_message (OUT_OF_MEMORY); 
    return (FALSE); 
  } 
  for (q = buffers[b_index].head->next; q; q = next_b) 
  {                                                 /* 1. for start */ 
 
    if (q == NULL) break;                   /* maybe there was only */ 
                                            /* one line to paste    */ 
    next_b = q->next; 
    if ((new = create_list_element ()) == NULL) 
    { 
      error = TRUE; 
      break;                                  /* leave the for loop */ 
    } 
    if ((new->line = (char *) malloc (MAX_LINE + 1)) == NULL) 
    { 
      error = TRUE; 
      free ((LINE_DESC *) new); 
      break;                                  /* leave the for loop */ 
    } 
    memset (new->line, (int) 0, (size_t) (MAX_LINE-1)); 
    strcpy (new->line, q->line);        /* here the paste work    */ 
                                        /* gets done              */ 
    new->len  = (int) strlen (new->line); 
    new->size = MAX_LINE; 
    new->prev = p; 
    new->next = p->next; 
    if (p->next) p->next->prev = new; 
    p->next = new; 
    p = new; 
  }                                                  /* 1. for end */ 
  if (error) 
  { 
    put_message (OUT_OF_MEMORY); 
    return (FALSE); 
  } 
 
  /* deal with the last line seperately */ 

  if ((p->len + strlen (gbl_line_buf1) < p->size - 2) &&
       p->line[p->len-1] != '\n') 
                                  /* Enough space left for copy ?   */
                                  /* If not, insert a CR.           */ 
  { 
    strcpy (p->line + p->len, gbl_line_buf1); 
    p->len = (int) strlen (p->line); 
  } 
  else 
  { 
    if ((new = create_list_element ()) == NULL) 
    { 
      error = TRUE; 
    } 
    if ((new->line = (char *) malloc (MAX_LINE + 1)) == NULL) 
    { 
      error = TRUE; 
      free ((LINE_DESC *) new); 
    } 
    memset (new->line, (int) 0, (size_t) (MAX_LINE-1));  
    strcpy (new->line, gbl_line_buf1);  /* copy the final line    */ 
    new->len  = (int) strlen (new->line); 
    new->size = MAX_LINE; 
    new->prev = p; 
    new->next = p->next; 
    if (p->next) p->next->prev = new; 
    p->next = new; 
    p = new; 
  } 
  if (error) 
  { 
    put_message (OUT_OF_MEMORY); 
    return (FALSE); 
  } 
  buffers[b_index].used = TRUE; 
 
  start_x = windows[w_index].doc_x / bzeich; 
  start_y = windows[w_index].doc_y / hzeich; 
  wind_w  = windows[w_index].work.g_w / bzeich; 
  wind_h  = windows[w_index].work.g_h / hzeich; 
 
  cursor_visible = TRUE; 
  if ((windows[w_index].curs_y < start_y + 1)       || 
      (windows[w_index].curs_y > start_y + wind_h)  || 
      (windows[w_index].curs_x < start_x + 1)       || 
      (windows[w_index].curs_x > start_x + wind_w)) 
  { 
      cursor_visible = FALSE; 
  } 
  if (!cursor_visible) 
  { 
    curs_y  = (windows[w_index].curs_y - 1) * hzeich; 
    doc_h   = windows[w_index].work.g_h; 
    windows[w_index].doc_y = 
      max ((long) (curs_y - doc_h/2), (long) 0); 
 
    curs_x  = (windows[w_index].curs_x - 1) * bzeich; 
    doc_w   = windows[w_index].work.g_w; 
    windows[w_index].doc_x = 
      max ((long) (curs_x - doc_w/2), (long) 0); 
  } 
  return (TRUE); 
}                                     /* end function paste_block */ 
 
 
 
int   paste_include_file (w_index) 
int   w_index;            /* index into global "windows" structure  */ 
{ 
  long          i; 
  long          error; 
  long          start_x, start_y, wind_h, wind_w; 
  long          doc_w, doc_h; 
  long          curs_x, curs_y, cursor_visible; 
  LINE_DESC     *p, *q, *next_b, *new; 
 
 
  error = FALSE; 
 
  for (i = 1, p = windows[w_index].head; 
       i < windows[w_index].curs_y; i++) 
  { 
    p = p->next; 
  } 
 
  start_x = windows[w_index].curs_x - 1; 
  memset (gbl_line_buf2, (int) 0, (size_t) (3*MAX_LINE-1));
  strcpy (gbl_line_buf2, (p->line) + start_x);  
                                          /* save rest of line    */
  strcpy (p->line + start_x, "\n");       /* truncate rest of line  */
  p->len = (int) strlen (p->line); 
  memset (p->line + p->len + 1, 
          (int) 0, (size_t) (p->size - p->len - 2));
 
  /* deal with the first line seperately  */ 
 
  q = include_buffer.head; 
  if (p->len + q->len < p->size - 2)     /* enough space for copy ? */ 
  { 
    strcpy (p->line + start_x, q->line); 
    p->len = (int) strlen (p->line); 
  } 
  else 
  { 
    if ((new = create_list_element ()) == NULL) 
    { 
      error = TRUE; 
    } 
    if ((new->line = (char *) malloc (MAX_LINE + 1)) == NULL) 
    { 
      error = TRUE; 
      free ((LINE_DESC *) new); 
    } 
    memset (new->line, (int) 0, (size_t) (MAX_LINE-1));
    strcpy (new->line, q->line);        /* here the paste work     */ 
                                        /* gets done               */ 
    new->len    = (int) strlen (new->line); 
    new->size   = MAX_LINE; 
    new->prev   = p; 
    new->next   = p->next; 
    if (p->next) p->next->prev = new; 
    p->next = new; 
    p = new; 
  } 
  if (error) 
  { 
    put_message (OUT_OF_MEMORY); 
    return (FALSE); 
  } 
  for (q = include_buffer.head->next; q; q = next_b) 
  {                                                 /* 1. for start */ 
 
    if (q == NULL) break;                   /* maybe there was only */ 
                                            /* one line to paste    */ 
    next_b = q->next; 
    if ((new = create_list_element ()) == NULL) 
    { 
      error = TRUE; 
      break;                                  /* leave the for loop */ 
    } 
    if ((new->line = (char *) malloc (MAX_LINE + 1)) == NULL) 
    { 
      error = TRUE; 
      free ((LINE_DESC *) new); 
      break;                                  /* leave the for loop */ 
    } 
    memset (new->line, (int) 0, (size_t) (MAX_LINE-1));  
    strcpy (new->line, q->line);        /* here the paste work    */ 
                                        /* gets done              */ 
    new->len  = (int) strlen (new->line); 
    new->size = MAX_LINE; 
    new->prev = p; 
    new->next = p->next; 
    if (p->next) p->next->prev = new; 
    p->next = new; 
    p = new; 
  }                                                  /* 1. for end */ 
  if (error) 
  { 
    put_message (OUT_OF_MEMORY); 
    return (FALSE); 
  } 
 
  /* deal with the last line seperately */ 

  if ((p->len + strlen (gbl_line_buf2) < p->size - 2) &&
       p->line[p->len-1] != '\n') 
                                  /* Enough space left for copy ?   */
                                  /* If not, insert a CR.           */ 
  { 
    strcpy (p->line + p->len, gbl_line_buf2); 
    p->len = (int) strlen (p->line); 
  } 
  else 
  { 
    if ((new = create_list_element ()) == NULL) 
    { 
      error = TRUE; 
    } 
    if ((new->line = (char *) malloc (MAX_LINE + 1)) == NULL) 
    { 
      error = TRUE; 
      free ((LINE_DESC *) new); 
    } 
    memset (new->line, (int) 0, (size_t) (MAX_LINE-1)); 
    strcpy (new->line, gbl_line_buf2); /* copy the final line  */ 
    new->len  = (int) strlen (new->line); 
    new->size = MAX_LINE; 
    new->prev = p; 
    new->next = p->next; 
    if (p->next) p->next->prev = new; 
    p->next = new; 
    p = new; 
  } 
  if (error) 
  { 
    put_message (OUT_OF_MEMORY); 
    return (FALSE); 
  } 
  include_buffer.used = TRUE; 
 
  start_x = windows[w_index].doc_x / bzeich; 
  start_y = windows[w_index].doc_y / hzeich; 
  wind_w  = windows[w_index].work.g_w / bzeich; 
  wind_h  = windows[w_index].work.g_h / hzeich; 
 
  cursor_visible = TRUE; 
  if ((windows[w_index].curs_y < start_y + 1)       || 
      (windows[w_index].curs_y > start_y + wind_h)  || 
      (windows[w_index].curs_x < start_x + 1)       || 
      (windows[w_index].curs_x > start_x + wind_w)) 
  { 
      cursor_visible = FALSE; 
  } 
  if (!cursor_visible) 
  { 
    curs_y  = (windows[w_index].curs_y - 1) * hzeich; 
    doc_h   = windows[w_index].work.g_h; 
    windows[w_index].doc_y = 
      max ((long) (curs_y - doc_h/2), (long) 0); 
 
    curs_x  = (windows[w_index].curs_x - 1) * bzeich; 
    doc_w   = windows[w_index].work.g_w; 
    windows[w_index].doc_x = 
      max ((long) (curs_x - doc_w/2), (long) 0); 
  } 
  return (TRUE); 
}                            /* end function paste_include_file */ 
 
 
 
 
int   init_buffer (b_index) 
int   b_index; 
{ 
  LINE_DESC   *q, *next; 
  TEDINFO     *ted_info; 
 
 
  if (b_index >= WD_MAX_BUFFERS) return (FALSE); 
  if (b_index < 0) return (FALSE); 
  
  if (buffers[b_index].head) 
  { 
    for (q = buffers[b_index].head; q; q = next) 
    { 
      next = q->next; 
      memset (q->line, (int) 0, (size_t) (MAX_LINE-1));
      free ((char *) q->line);
      q->line   = NULL; 
      q->prev   = NULL; 
      q->next   = NULL; 
      q->len    = 0; 
      q->size   = 0; 
      free ((LINE_DESC *) q);           /* free allocated memory   */ 
    } 
    buffers[b_index].head = NULL; 
  } 
  buffers[b_index].used = FALSE; 
  menu_ienable (menu_tree, ICLBUFF,  FALSE); 
  switch (b_index)
  {
    case 0:   ted_info = dial_selebuf[BUFNAM1].ob_spec.tedinfo; 
              break;

    case 1:   ted_info = dial_selebuf[BUFNAM2].ob_spec.tedinfo; 
              break;

    case 2:   ted_info = dial_selebuf[BUFNAM3].ob_spec.tedinfo; 
              break;

    case 3:   ted_info = dial_selebuf[BUFNAM4].ob_spec.tedinfo; 
              break;

    case 4:   ted_info = dial_selebuf[BUFNAM5].ob_spec.tedinfo; 
              break;

    case 5:   ted_info =  dial_selebuf[BUFNAM6].ob_spec.tedinfo; 
              break;

    case 6:   ted_info = dial_selebuf[BUFNAM7].ob_spec.tedinfo; 
              break;

    case 7:   ted_info =  dial_selebuf[BUFNAM8].ob_spec.tedinfo; 
              break;

    case 8:   ted_info = dial_selebuf[BUFNAM9].ob_spec.tedinfo; 
              break;

    case 9:   ted_info = dial_selebuf[BUFNAM10].ob_spec.tedinfo; 
              break;

    default:  say ("Unexpected buffer error"); 
              break;
  }
  strcpy (ted_info->te_ptext, "");
  return (TRUE); 
}                                      /* end function init_buffer */ 
 
 
 
int   init_include_buffer () 
{ 
  LINE_DESC   *q, *next; 
 
 
  if (include_buffer.head) 
  { 
    for (q = include_buffer.head; q; q = next) 
    { 
      next = q->next; 
      memset (q->line, (int) 0, (size_t) (MAX_LINE-1));
      free ((char *) q->line);
      q->line   = NULL; 
      q->prev   = NULL; 
      q->next   = NULL; 
      q->len    = 0; 
      q->size   = 0; 
      free ((LINE_DESC *) q);           /* free allocated memory   */ 
    } 
    include_buffer.head = NULL; 
  } 
  include_buffer.used = FALSE; 
  return (TRUE); 
}                              /* end function init_include_buffer */ 
 
 
 
 
int   exist_buffer () 
{ 

  if (buffers[buffer_index].head) return (TRUE); 
  return (FALSE); 
}                                    /* end function exist_buffer  */ 
 

void  clear_block_flags (w_index)
int   w_index;
{
  windows[w_index].block.set  = FALSE; 
  windows[w_index].block.cs_x = -1; 
  windows[w_index].block.cs_y = -1; 
  windows[w_index].block.cs_y = -1; 
  windows[w_index].block.ce_y = -1; 
  return;
}
 
 
 
/* end of file block.c */ 
 
