/* 
output.c 
ALICE V 2.0, January 1993
library functions for output handling 
(C) Ralf Kaufmann 1993
used C compiler: Pure C 
*/ 
 
/* 
functions contained in this module: 
output 
put_graphic_char 
cursor
display_pos 
beep
*/ 
 
/* Includefiles */ 
 
# include <ctype.h> 
# include <math.h> 
# include <stdio.h> 
# include <stdlib.h> 
# include <string.h> 
 
# include <aes.h> 
# include <tos.h> 
# include <vdi.h> 
 
 
# include "language.h" 
# include "prvdef.h" 
# include "mewidi.h" 
# include "messages.h" 
# include "scancode.h" 
# include "structs.h" 
# include "extdef.h" 
# include "rsc.h" 
 
/* References to global data defined elsewhere  */ 
 
extern  int         ap_id, handle; 
extern  int         bzeich, hzeich, gbl_clip[]; 
extern  WIND_DATA   windows[]; 
extern  BUFFER      buffers[]; 
extern  OBJECT      *menu_tree; 
extern  OBJECT      *dial_about, *dial_help, *dial_print, *dial_tab; 
extern  int         prgquit, prgexit, page_length, left_margin; 
extern  int         tab_size, showcr, showpos; 
extern  int         line_len, eol_warn; 
 
 
/**********************/ 
/* Start of routines  */ 
/**********************/ 
 
 
 
 
/* Function to display any text */ 
 
void  output (w_handle, area, clear)  /* output text on the screen  */
int   w_handle;                       /* window handle              */ 
GRECT *area;                          /* describes the redraw area  */ 
int   clear;                          /* clear before writing ?     */ 
{                                     /* start function output      */ 
 
  long        i, j; 
  int         x, y, w_index, letter[4]; 
  long        start_x, start_y, end_y; 
  LINE_DESC   *p; 
 
 
  w_index = get_index_from_handle (w_handle); 
  if (windows[w_index].head == NULL) return; /* nothing there   */ 
                                                 /* to be displayed */ 
  if (clear) 
  { 
    ALLWHITE((area->g_x),(area->g_y),(area->g_w),(area->g_h)); 
  } 

  start_x = windows[w_index].doc_x / bzeich;  /* char offset in doc */ 
  start_y = windows[w_index].doc_y / hzeich;  /* line offset in doc */ 
  end_y   = start_y + (windows[w_index].work.g_h / hzeich); 
 
  if (start_y > windows[w_index].doc_length) return; 
                                         /* no line to be displayed */ 
  area->g_h += area->g_y; 
  area->g_w += area->g_x; 
  area->g_x -= bzeich; 
  area->g_y -= hzeich; 
 
  for (i = 0, p = windows[w_index].head; i < start_y; i++) 
  { 
    p = p->next; 
  } 
 
  for (y = windows[w_index].work.g_y, i = start_y; 
       y < windows[w_index].work.g_y + windows[w_index].work.g_h && 
       p && i < end_y; 
       y += hzeich, p = p->next, i++) 
  {                                                 /* 1. for start */ 
    for (x = windows[w_index].work.g_x, j = start_x; 
         x < windows[w_index].work.g_x + windows[w_index].work.g_w && 
         j < p->len; 
         x += bzeich, j++) 
    {                                               /* 2. for start */ 
      if (y >= area->g_y && x >= area->g_x && 
          y <= area->g_h && x <= area->g_w )
      {                                             /* 1. if start  */ 
 
        /* block marks required ? */ 
 
        if  (!windows[w_index].block.set) 
        { 
          put_graphic_char (p->line[j], x, y, w_index);      
                                                    /* output char  */ 
          continue; 
        } 
        if ((i + 1 < windows[w_index].block.cs_y) || 
            (i + 1 > windows[w_index].block.ce_y)) 
        { 
          put_graphic_char (p->line[j], x, y, w_index);      
                                                    /* output char  */ 
          continue; 
        } 
 
        letter[0] = x; 
        letter[1] = y; 
        letter[2] = x + bzeich - 1; 
        letter[3] = y + hzeich - 1; 
 
        if ((i + 1 >  windows[w_index].block.cs_y) && 
            (i + 1 <  windows[w_index].block.ce_y)) 
        { 
          v_bar (handle, letter); 
          vswr_mode (handle, MD_TRANS); 
                                      /* write mode is transparent  */ 
        } 
        else if ((i + 1 == windows[w_index].block.cs_y) && 
                 (i + 1 == windows[w_index].block.ce_y) && 
                 (j + 1 >= windows[w_index].block.cs_x) && 
                 (j + 1 <  windows[w_index].block.ce_x)) 
        { 
          v_bar (handle, letter); 
          vswr_mode (handle, MD_TRANS); 
                                      /* write mode is transparent  */ 
        } 
        else if ((i + 1 == windows[w_index].block.cs_y) && 
                 (i + 1 <  windows[w_index].block.ce_y) && 
                 (j + 1 >= windows[w_index].block.cs_x)) 
        { 
          v_bar (handle, letter); 
          vswr_mode (handle, MD_TRANS); 
                                      /* write mode is transparent  */ 
        } 
        else if ((i + 1 == windows[w_index].block.ce_y) && 
                 (i + 1 >  windows[w_index].block.cs_y) && 
                 (j + 1 <  windows[w_index].block.ce_x)) 
        { 
          v_bar (handle, letter); 
          vswr_mode (handle, MD_TRANS); 
                                      /* write mode is transparent  */ 
        } 
        else if ((i + 1 == windows[w_index].block.ce_y) && 
                 (j + 1 == windows[w_index].block.ce_x)) 
        { 
          vswr_mode (handle, MD_REPLACE); 
                                   /* write mode is back to replace */ 
        } 
 
        put_graphic_char (p->line[j], x, y, w_index);        
                                                    /* output char  */ 
 
      }                                             /* 1. if end    */ 
    }                                               /* 2. for end   */ 
  } 
  v_hide_c (handle);                              /* hide the mouse */ 
  if (w_handle == top_wind ()) cursor (w_handle, 1); 
  v_show_c (handle, TRUE);                        /* show the mouse */
  MOUSE_IS_ARROW; 
  vswr_mode (handle, MD_REPLACE);  /* write mode is back to replace */ 
  return; 
}                                           /* end function output  */ 
 
 
 
void  put_graphic_char (c, x, y, w_index)   /* display char c    */ 
                                            /* at position x, y  */ 
char    c; 
size_t  x, y; 
int     w_index;
{ 
  static  char  one_char[2] = { ' ', '\0'}; 
 
  one_char[0] = c; 

  if (one_char[0] == '\n' && w_index != -1) 
  {
    if (showcr)  
    {
      one_char[0] = EOL_SYMBOL;
    }
    else
    {
      one_char[0] = ' ';
    }
  }

  v_gtext ((int) handle,              /* here's where the output  */ 
           (int) x, (int) y,          /* eventually finds its way */ 
           one_char);                 /* to the screen            */ 
 
}                               /* end function put_graphic_char  */ 
 
 
 
int   cursor (w_handle, mode)         /* display or hide text cursor */
int   w_handle, mode; 
{ 
  int   w_index, cursor_visible; 
  long  start_x, start_y; 
  int   help; 
  int   wind_w, wind_h; 
  int   x, y; 
 
  if (!own_top_wind ()) return (TRUE);
  if (top_wind () != w_handle) return (TRUE); 
  w_index = get_index_from_handle (w_handle); 
 
  start_x = windows[w_index].doc_x / bzeich; 
  start_y = windows[w_index].doc_y / hzeich; 
  wind_w  = windows[w_index].work.g_w / bzeich; 
                                /* number of letters in width ... */ 
  wind_h  = windows[w_index].work.g_h / hzeich; 
                                                /* ... and heigth */ 
 
  cursor_visible = TRUE; 
  if ((windows[w_index].curs_y < start_y + 1)       || 
      (windows[w_index].curs_y > start_y + wind_h)  || 
      (windows[w_index].curs_x < start_x + 1)       || 
      (windows[w_index].curs_x > start_x + wind_w)) 
  { 
    cursor_visible = FALSE; 
  } 
 
 
  if (!cursor_visible) return (TRUE); 
  if (mode != windows[w_index].curs_vis) 
  { 
    help  = (int) (windows[w_index].curs_x - 1 - start_x) * bzeich; 
    help  = (int) max ((long) help, (long) 0); 
    x     = windows[w_index].work.g_x + help; 
    help  = (int) (windows[w_index].curs_y - 1 - start_y) * hzeich; 
    help  = (int) max ((long) help, (long) 0); 
    y     = windows[w_index].work.g_y + help; 
    INVERT(x,y,(bzeich-1),(hzeich-1));    /* display/hide cursor  */ 
    windows[w_index].curs_vis ^= 1;           /* invert the flag  */ 
  }
  if (showpos && windows[w_index].curs_vis) display_pos (w_index);
  if ((mode != 1) || (top_wind () != w_handle)) return (TRUE);

  if (eol_warn == TRUE && 
      line_length (w_index, windows[w_index].curs_y) > line_len)
  {
    BELL;
  }
  return (TRUE); 
}                                       /* end of function cursor */ 
 

 
void  display_pos (w_index)   /* display cursor position in   */
int   w_index;                /* info line                    */
{
  int i, p;
  
  if (dial_print[HEADYES].ob_state == SELECTED)
  {
    p = (int) (page_length - 2);
  }
  else
  {
    p = (int) page_length;
  }
  i = (int) ((windows[w_index].curs_y - 1)/ p);
  i++;
   
  if (showpos)
  {

# ifdef english
    sprintf (windows[w_index].info_line, 
              " Page %d, Line %ld (%ld), Column %ld",
              i, 
              windows[w_index].curs_y - (i-1)*p,
              windows[w_index].curs_y, 
              windows[w_index].curs_x);
# endif
# ifdef german
    sprintf (windows[w_index].info_line, 
              " Seite %d, Zeile %ld (%ld), Spalte %ld",
              i,
              windows[w_index].curs_y - (i-1)*p, 
              windows[w_index].curs_y, 
              windows[w_index].curs_x);
# endif
# ifdef french
    sprintf (windows[w_index].info_line, 
              " Page %d, Ligne %ld (%ld), Colonne %ld",
              i,
              windows[w_index].curs_y - (i-1)*p, 
              windows[w_index].curs_y, 
              windows[w_index].curs_x);
# endif

  }
  else
  {
    strcpy (windows[w_index].info_line, "");
  }
  wind_set (windows[w_index].handle, WF_INFO,   /* set info line */ 
            HIGHW(windows[w_index].info_line), 
            LOWW(windows[w_index].info_line), 
            0, 0);
  return;
}



/* end of file output.c  */ 
 
